@@ -11,16 +11,13 @@ Created on 2018年07月18日
 """
 
 
-import requests
 import json
 from bs4 import BeautifulSoup
-import urllib
 from io import StringIO
-import pandas as pd
 import datetime
-from fushare import cons
+import warnings
+from fushare.requests_fun import *
 from fushare.symbolVar import *
-from fushare.requests_fun import requests_link
 calendar = cons.get_calendar()
 
 rank_columns = ['vol_party_name', 'vol', 'vol_chg','long_party_name', 'long_openIntr',
@@ -62,9 +59,13 @@ def get_rank_sum_daily(start=None, end=None, vars=cons.vars):
     while start <= end:
         print(start)
         if start.strftime('%Y%m%d') in calendar:
-            records = records.append(get_rank_sum(start, vars))
+            data = get_rank_sum(start, vars)
+            if data is False:
+                print('%s日交易所数据连接失败，已超过20次，您的地址被网站墙了，请保存好返回数据，稍后从该日期起重试' % start.strftime('%Y-%m-%d'))
+                return records.reset_index(drop=True)
+            records = records.append(data)
         else:
-            print('%s非交易日' % start.strftime('%Y%m%d'))
+            warnings.warn('%s非交易日' % start.strftime('%Y%m%d'))
         start += datetime.timedelta(days=1)
 
     return records.reset_index(drop=True)
@@ -101,6 +102,7 @@ def get_rank_sum(date = None,vars=cons.vars):
     """
     date = cons.convert_date(date) if date is not None else datetime.date.today()
     if date.strftime('%Y%m%d') not in calendar:
+        warnings.warn('%s非交易日' % date.strftime('%Y%m%d'))
         return None
     dce_var = [i for i in vars if i in cons.market_var['dce']]
     shfe_var = [i for i in vars if i in cons.market_var['shfe']]
@@ -108,13 +110,25 @@ def get_rank_sum(date = None,vars=cons.vars):
     cffex_var = [i for i in vars if i in cons.market_var['cffex']]
     D={}
     if len(dce_var)>0:
-        D.update(get_dce_rank_table(date, dce_var))
+        data = get_dce_rank_table(date, dce_var)
+        if data is False:
+            return False
+        D.update(data)
     if len(shfe_var)>0:
-        D.update(get_shfe_rank_table(date, shfe_var))
+        data = get_shfe_rank_table(date, shfe_var)
+        if data is False:
+            return False
+        D.update(data)
     if len(czce_var)>0:
-        D.update(get_czce_rank_table(date, czce_var))
+        data = get_czce_rank_table(date, czce_var)
+        if data is False:
+            return False
+        D.update(data)
     if len(cffex_var)>0:
-        D.update(get_cffex_rank_table(date, cffex_var))
+        data = get_cffex_rank_table(date, cffex_var)
+        if data is False:
+            return False
+        D.update(data)
     records=pd.DataFrame()
 
 
@@ -123,49 +137,51 @@ def get_rank_sum(date = None,vars=cons.vars):
         for symbol in set(table['symbol']):
 
             var = symbol2varietie(symbol)
-            tableCut = table[table['symbol'] == symbol]
-            tableCut_top5 = tableCut[tableCut['rank'] <= 5]
-            tableCut_top10 = tableCut[tableCut['rank'] <= 10]
-            tableCut_top15 = tableCut[tableCut['rank'] <= 15]
-            tableCut_top20 = tableCut[tableCut['rank'] <= 20]
+            if var in vars:
+                tableCut = table[table['symbol'] == symbol]
+                tableCut['rank'] = tableCut['rank'].astype('float')
+                tableCut_top5 = tableCut[tableCut['rank'] <= 5]
+                tableCut_top10 = tableCut[tableCut['rank'] <= 10]
+                tableCut_top15 = tableCut[tableCut['rank'] <= 15]
+                tableCut_top20 = tableCut[tableCut['rank'] <= 20]
 
-            D = {'symbol': symbol, 'var': var,
+                D = {'symbol': symbol, 'variety': var,
 
-                 'vol_top5': tableCut_top5['vol'].sum(), 'vol_chg_top5': tableCut_top5['vol_chg'].sum(),
-                 'long_openIntr_top5': tableCut_top5['long_openIntr'].sum(),
-                 'long_openIntr_chg_top5': tableCut_top5['long_openIntr_chg'].sum(),
-                 'short_openIntr_top5': tableCut_top5['short_openIntr'].sum(),
-                 'short_openIntr_chg_top5': tableCut_top5['short_openIntr_chg'].sum(),
+                     'vol_top5': tableCut_top5['vol'].sum(), 'vol_chg_top5': tableCut_top5['vol_chg'].sum(),
+                     'long_openIntr_top5': tableCut_top5['long_openIntr'].sum(),
+                     'long_openIntr_chg_top5': tableCut_top5['long_openIntr_chg'].sum(),
+                     'short_openIntr_top5': tableCut_top5['short_openIntr'].sum(),
+                     'short_openIntr_chg_top5': tableCut_top5['short_openIntr_chg'].sum(),
 
-                 'vol_top10': tableCut_top10['vol'].sum(), 'vol_chg_top10': tableCut_top10['vol_chg'].sum(),
-                 'long_openIntr_top10': tableCut_top10['long_openIntr'].sum(),
-                 'long_openIntr_chg_top10': tableCut_top10['long_openIntr_chg'].sum(),
-                 'short_openIntr_top10': tableCut_top10['short_openIntr'].sum(),
-                 'short_openIntr_chg_top10': tableCut_top10['short_openIntr_chg'].sum(),
+                     'vol_top10': tableCut_top10['vol'].sum(), 'vol_chg_top10': tableCut_top10['vol_chg'].sum(),
+                     'long_openIntr_top10': tableCut_top10['long_openIntr'].sum(),
+                     'long_openIntr_chg_top10': tableCut_top10['long_openIntr_chg'].sum(),
+                     'short_openIntr_top10': tableCut_top10['short_openIntr'].sum(),
+                     'short_openIntr_chg_top10': tableCut_top10['short_openIntr_chg'].sum(),
 
-                 'vol_top15': tableCut_top15['vol'].sum(), 'vol_chg_top15': tableCut_top15['vol_chg'].sum(),
-                 'long_openIntr_top15': tableCut_top15['long_openIntr'].sum(),
-                 'long_openIntr_chg_top15': tableCut_top15['long_openIntr_chg'].sum(),
-                 'short_openIntr_top15': tableCut_top15['short_openIntr'].sum(),
-                 'short_openIntr_chg_top15': tableCut_top15['short_openIntr_chg'].sum(),
+                     'vol_top15': tableCut_top15['vol'].sum(), 'vol_chg_top15': tableCut_top15['vol_chg'].sum(),
+                     'long_openIntr_top15': tableCut_top15['long_openIntr'].sum(),
+                     'long_openIntr_chg_top15': tableCut_top15['long_openIntr_chg'].sum(),
+                     'short_openIntr_top15': tableCut_top15['short_openIntr'].sum(),
+                     'short_openIntr_chg_top15': tableCut_top15['short_openIntr_chg'].sum(),
 
-                 'vol_top20': tableCut_top20['vol'].sum(), 'vol_chg_top20': tableCut_top20['vol_chg'].sum(),
-                 'long_openIntr_top20': tableCut_top20['long_openIntr'].sum(),
-                 'long_openIntr_chg_top20': tableCut_top20['long_openIntr_chg'].sum(),
-                 'short_openIntr_top20': tableCut_top20['short_openIntr'].sum(),
-                 'short_openIntr_chg_top20': tableCut_top20['short_openIntr_chg'].sum(),
+                     'vol_top20': tableCut_top20['vol'].sum(), 'vol_chg_top20': tableCut_top20['vol_chg'].sum(),
+                     'long_openIntr_top20': tableCut_top20['long_openIntr'].sum(),
+                     'long_openIntr_chg_top20': tableCut_top20['long_openIntr_chg'].sum(),
+                     'short_openIntr_top20': tableCut_top20['short_openIntr'].sum(),
+                     'short_openIntr_chg_top20': tableCut_top20['short_openIntr_chg'].sum(),
 
-                 'date': date.strftime('%Y%m%d')
-                 }
-            records = records.append(pd.DataFrame(D, index=[0]))
+                     'date': date.strftime('%Y%m%d')
+                     }
+                records = records.append(pd.DataFrame(D, index=[0]))
 
     if len(D.items())>0:
-        add_vars = [i for i in cons.market_var['shfe']+cons.market_var['cffex'] if i in records['var'].tolist()]
+        add_vars = [i for i in cons.market_var['shfe']+cons.market_var['cffex'] if i in records['variety'].tolist()]
         for var in add_vars:
-            recordsCut = records[records['var'] == var]
+            recordsCut = records[records['variety'] == var]
             var_record = pd.DataFrame(recordsCut.sum()).T
             var_record['date'] = date.strftime('%Y%m%d')
-            var_record.loc[:,['var','symbol']] = var
+            var_record.loc[:,['variety','symbol']] = var
             records = records.append(var_record)
 
     return records.reset_index(drop=True)
@@ -197,8 +213,11 @@ def get_shfe_rank_table(date = None,vars = cons.vars):
                 date                        日期                        string YYYYMMDD
     """
     date = cons.convert_date(date) if date is not None else datetime.date.today()
+    if date < datetime.date(2002,1,7):
+        print("shfe数据源开始日期为20020107，跳过")
+        return {}
     if date.strftime('%Y%m%d') not in calendar:
-        print('%s非交易日' % date.strftime('%Y%m%d'))
+        warnings.warn('%s非交易日' % date.strftime('%Y%m%d'))
         return {}
     url = cons.SHFE_VOLRANK_URL %(date.strftime('%Y%m%d'))
     r = requests_link(url,'utf-8')
@@ -219,7 +238,7 @@ def get_shfe_rank_table(date = None,vars = cons.vars):
         return {}
     df = df.applymap(lambda x: x.strip() if type(x) == type('') else x)
     df = df.applymap(lambda x: None if x == '' else x)
-    df['var'] = df['symbol'].apply(lambda x: symbol2varietie(x))
+    df['variety'] = df['symbol'].apply(lambda x: symbol2varietie(x))
 
     df = df[df['rank'] > 0]
     for col in ['PARTICIPANTID1','PARTICIPANTID2','PARTICIPANTID3','product1','product2']:
@@ -227,10 +246,10 @@ def get_shfe_rank_table(date = None,vars = cons.vars):
             del df[col]
         except:
             pass
-    get_vars = [var for var in vars if var in df['var'].tolist()]
+    get_vars = [var for var in vars if var in df['variety'].tolist()]
     D={}
     for var in get_vars:
-        df_var = df[df['var'] == var]
+        df_var = df[df['variety'] == var]
         for symbol in set(df_var['symbol']):
             df_symbol = df_var[df_var['symbol'] == symbol]
             D[symbol] = df_symbol.reset_index(drop=True)
@@ -252,7 +271,7 @@ def _czce_df_read(url,skiprow,encode='utf-8'):
     """
     r = requests_link(url,encode)
     data = pd.read_html(r.text, match='.+', flavor=None, header=0, index_col=0, skiprows=skiprow, attrs=None,
-                        parse_dates=False, tupleize_cols=False, thousands=', ', encoding="gbk", decimal='.',
+                        parse_dates=False,  thousands=', ', encoding="gbk", decimal='.',
                         converters=None, na_values=None, keep_default_na=True)
     return data
 
@@ -283,8 +302,11 @@ def get_czce_rank_table(date = None,vars = cons.vars):
                 date                        日期                        string YYYYMMDD
     """
     date = cons.convert_date(date) if date is not None else datetime.date.today()
+    if date < datetime.date(2005,5,9):
+        print("czce数据源开始日期为20050509，跳过")
+        return {}
     if date.strftime('%Y%m%d') not in calendar:
-        print('%s非交易日' % date.strftime('%Y%m%d'))
+        warnings.warn('%s非交易日' % date.strftime('%Y%m%d'))
         return {}
     if date <= datetime.date(2010, 8, 25):
         url = cons.CZCE_VOLRANK_URL_1 % (date.strftime('%Y%m%d'))
@@ -304,14 +326,21 @@ def get_czce_rank_table(date = None,vars = cons.vars):
         D={}
         for i in range(len(symbols)):
             symbol = symbols[i]
-            tableCut = data[i+1]
+            tableCut = data[i+2]
             tableCut.columns = rank_columns
             tableCut = tableCut.iloc[:-1,:]
             tableCut.loc[:,'rank'] = tableCut.index
             tableCut.loc['合计','rank'] = 999
             tableCut.loc['合计',['vol_party_name','long_party_name','short_party_name']] = None
             tableCut.loc[:,'symbol'] = symbol
-            tableCut.loc[:,'var'] = symbol2varietie(symbol)
+            tableCut.loc[:,'variety'] = symbol2varietie(symbol)
+            tableCut[intColumns] = tableCut[intColumns].fillna(0)
+            tableCut[intColumns] = tableCut[intColumns].astype(str)
+            tableCut[intColumns] = tableCut[intColumns].applymap(lambda x: x.replace(',', ''))
+            tableCut = tableCut.applymap(lambda x: 0 if x == '-' else x)
+
+            tableCut[intColumns] = tableCut[intColumns].astype(float)
+            tableCut[intColumns] = tableCut[intColumns].astype(int)
             D[symbol] = tableCut.reset_index(drop=True)
         return D
 
@@ -328,7 +357,7 @@ def get_czce_rank_table(date = None,vars = cons.vars):
     if len(data.columns) <6:
         return {}
 
-    table = data.iloc[:, :9]
+    table = pd.DataFrame(data.iloc[:, :9])
     table.columns = rank_columns
     table.loc[:,'rank'] = table.index
     table[intColumns] = table[intColumns].astype(str)
@@ -339,6 +368,7 @@ def get_czce_rank_table(date = None,vars = cons.vars):
     D = {}
 
     for i in range(len(indexs)):
+
         if indexs[i] == 0:
             tableCut = table.loc[:indexs[i + 1], :]
             string = tableCut.index.name
@@ -346,6 +376,7 @@ def get_czce_rank_table(date = None,vars = cons.vars):
         elif i < len(indexs) - 1:
             tableCut = table.loc[indexs[i]:indexs[i + 1], :]
             string = tableCut.index[0]
+
         else:
             tableCut = table.loc[indexs[i]:, :]
             string = tableCut.index[0]
@@ -357,12 +388,17 @@ def get_czce_rank_table(date = None,vars = cons.vars):
                 symbol = chinese_to_english(find_chinese(re.compile('：(.*) ').findall(string)[0]))
             except:
                 symbol = re.compile('：(.*) ').findall(string)[0]
+
         var = symbol2varietie(symbol)
+
         if var in vars:
+
             tableCut = tableCut.dropna(how='any').iloc[1:, :]
             tableCut = tableCut.loc[[x for x in tableCut.index if x in [str(i) for i in range(21)]], :]
+
             tableCut = _tableCut_cal(tableCut, symbol)
             D[symbol] = tableCut.reset_index(drop=True)
+
     return D
 
 
@@ -392,10 +428,12 @@ def get_dce_rank_table(date = None,vars = cons.vars):
                 var                         品种                        string
                 date                        日期                        string YYYYMMDD
     """
-	
     date = cons.convert_date(date) if date is not None else datetime.date.today()
+    if date < datetime.date(2006, 1, 4):
+        print(Exception("dce数据源开始日期为20060104，跳过"))
+        return {}
     if date.strftime('%Y%m%d') not in calendar:
-        print('%s非交易日' % date.strftime('%Y%m%d'))
+        warnings.warn('%s非交易日' % date.strftime('%Y%m%d'))
         return {}
     vars = [i for i in vars if i in cons.market_var['dce']]
     D={}
@@ -406,7 +444,10 @@ def get_dce_rank_table(date = None,vars = cons.vars):
         list_60 = []
         list_60_chg = []
         rank = []
-        texts = urllib.request.urlopen(url).readlines()
+
+        texts = urllib_request_link(url)
+        if texts == None:
+            return False
         if len(texts)>30:
             for text in texts:
                 line = text.decode('utf8')
@@ -432,7 +473,7 @@ def get_dce_rank_table(date = None,vars = cons.vars):
                                })
             tableCut = tableCut.applymap(lambda x: x.replace(',', ''))
             tableCut = _tableCut_cal(tableCut, var)
-            D[var] = tableCut
+            D[var] = tableCut.reset_index(drop=True)
     return D
 
 def get_cffex_rank_table(date = None,vars = cons.vars):
@@ -463,13 +504,18 @@ def get_cffex_rank_table(date = None,vars = cons.vars):
     """
     vars = [i for i in vars if i in cons.market_var['cffex']]
     date = cons.convert_date(date) if date is not None else datetime.date.today()
+    if date < datetime.date(2010,4,16):
+        print(Exception("cffex数据源开始日期为20100416，跳过"))
+        return {}
     if date.strftime('%Y%m%d') not in calendar:
-        print('%s非交易日' % date.strftime('%Y%m%d'))
+        warnings.warn('%s非交易日' % date.strftime('%Y%m%d'))
         return {}
     D={}
     for var in vars:
         url = cons.CFFEX_VOLRANK_URL % (date.strftime('%Y%m'), date.strftime('%d'), var)
         r = requests_link(url,encoding='gbk')
+        if r == None:
+            return False
         if '网页错误' not in r.text:
             table = pd.read_csv(StringIO(r.text.split('\n交易日,')[1]))
             table = table.dropna(how='any')
@@ -477,7 +523,7 @@ def get_cffex_rank_table(date = None,vars = cons.vars):
             for symbol in set(table['合约']):
                 tableCut =table[table['合约'] == symbol]
                 tableCut.columns = ['symbol','rank']+rank_columns
-                tableCut = _tableCut_cal(tableCut,symbol)
+                tableCut = _tableCut_cal(pd.DataFrame(tableCut),symbol)
                 D[symbol] = tableCut.reset_index(drop=True)
     return D
 
@@ -488,12 +534,21 @@ def _tableCut_cal(tableCut, symbol):
     tableCut_sum['rank'] = 999
     for col in ['vol_party_name', 'long_party_name', 'short_party_name']:
         tableCut_sum[col] = None
-    tableCut = tableCut.append(pd.DataFrame(tableCut_sum).T)
+    tableCut = tableCut.append(pd.DataFrame(tableCut_sum).T,sort = True)
     tableCut['symbol'] = symbol
-    tableCut['var'] = var
+    tableCut['variety'] = var
+
     return tableCut
 
 
 if __name__ == '__main__':
-    df = get_rank_sum_daily(start='20180905')
-    print(df)
\ No newline at end of file
+    df = get_rank_sum('20181105')
+    print(df)
+    #df = get_czce_rank_table(date = '20181105')
+    #print(df)
+    #df = get_dce_rank_table(date = '20181105')
+    #print(df)
+    df = get_shfe_rank_table(date = '20181105')
+    print(df)
+    df = get_cffex_rank_table(date = '20181105')
+    print(df)
