@@ -13,6 +13,8 @@ from bs4 import BeautifulSoup
 import pandas as pd
 from fushare import cons
 import requests
+import warnings
+
 calendar = cons.get_calendar()
 
 try:
@@ -53,7 +55,7 @@ def get_cffex_daily(date=None):
     """
     day = cons.convert_date(date) if date is not None else datetime.date.today()
     if day.strftime('%Y%m%d') not in calendar:
-        print('%s非交易日' %date.strftime('%Y%m%d'))
+        warnings.warn('%s非交易日' %day.strftime('%Y%m%d'))
         return None
     try:
         html = urlopen(Request(cons.CFFEX_DAILY_URL % (day.strftime('%Y%m'),
@@ -86,7 +88,10 @@ def get_cffex_daily(date=None):
             elif field in ['volume', 'open_interest', 'oi_chg']:
                 row_dict[field] = int(row[i + 1])
             else:
-                row_dict[field] = float(row[i + 1])
+                try:
+                    row_dict[field] = float(row[i + 1])
+                except:
+                    pass
         row_dict['pre_settle'] = row_dict['close'] - row_dict['change1']
         dict_data.append(row_dict)
 
@@ -100,7 +105,6 @@ def get_czce_daily(date=None):
     Parameters
     ------
         date: 日期 format：YYYY-MM-DD 或 YYYYMMDD 或 datetime.date对象 为空时为当天
-        type: 数据类型, 为'future'期货 或 'option'期权二者之一
     Return
     -------
         DataFrame
@@ -140,7 +144,7 @@ def get_czce_daily(date=None):
     """
     day = cons.convert_date(date) if date is not None else datetime.date.today()
     if day.strftime('%Y%m%d') not in calendar:
-        print('%s非交易日' %date.strftime('%Y%m%d'))
+        warnings.warn('%s非交易日' %day.strftime('%Y%m%d'))
         return None
     if day > datetime.date(2010, 8, 24):
         if day > datetime.date(2015,9,19):
@@ -255,13 +259,13 @@ def get_shfe_vwap(date=None):
     """
     day = cons.convert_date(date) if date is not None else datetime.date.today()
     if day.strftime('%Y%m%d') not in calendar:
-        print('%s非交易日' %date.strftime('%Y%m%d'))
+        warnings.warn('%s非交易日' %day.strftime('%Y%m%d'))
         return None
     try:
         json_data = json.loads(urlopen(Request(cons.SHFE_VWAP_URL % (day.strftime('%Y%m%d')),
                                                headers=cons.headers)).read().decode('utf8'))
     except HTTPError as reason:
-        if reason.code != 404:
+        if reason.code not in [404,403]:
             print(cons.SHFE_DAILY_URL % (day.strftime('%Y%m%d')), reason)
         return None
 
@@ -302,7 +306,7 @@ def get_shfe_daily(date=None):
     """
     day = cons.convert_date(date) if date is not None else datetime.date.today()
     if day.strftime('%Y%m%d') not in calendar:
-        print('%s非交易日' %date.strftime('%Y%m%d'))
+        warnings.warn('%s非交易日' %day.strftime('%Y%m%d'))
         return None
     try:
         json_data = json.loads(urlopen(Request(cons.SHFE_DAILY_URL % (day.strftime('%Y%m%d')),
@@ -316,7 +320,7 @@ def get_shfe_daily(date=None):
         return
 
     df = pd.DataFrame(
-        [row for row in json_data['o_curinstrument'] if row['DELIVERYMONTH'] != u'小计' and row['DELIVERYMONTH'] != ''])
+        [row for row in json_data['o_curinstrument'] if row['DELIVERYMONTH'] not in [u'小计',u'合计'] and row['DELIVERYMONTH'] != ''])
     df['variety'] = df.PRODUCTID.str.slice(0, -6).str.upper()
     df['symbol'] = df['variety'] + df['DELIVERYMONTH']
     df['date'] = day.strftime('%Y%m%d')
@@ -377,7 +381,7 @@ def get_dce_daily(date=None, type="future", retries=0):
     """
     day = cons.convert_date(date) if date is not None else datetime.date.today()
     if day.strftime('%Y%m%d') not in calendar:
-        print('%s非交易日' %date.strftime('%Y%m%d'))
+        warnings.warn('%s非交易日' %day.strftime('%Y%m%d'))
         return None
     if retries > 3:
         print("maximum retires for DCE market data: ", day.strftime("%Y%m%d"))
@@ -516,5 +520,5 @@ def get_future_daily(start=None, end=None, market='CFFEX'):
         return pd.concat(df_list)
 
 if __name__ == '__main__':
-    d = get_future_daily('20180906')
+    d = get_shfe_daily('20181126')
     print(d)
\ No newline at end of file
