@@ -10,12 +10,10 @@ Created on 2018年07月12日
 
 """
 
-import requests
+
 import json
-import re
-import pandas as pd
 import datetime
-from fushare import cons
+import warnings
 from fushare.symbolVar import *
 calendar = cons.get_calendar()
 from fushare.requests_fun import *
@@ -40,7 +38,7 @@ def get_dce_reciept(date = None,vars=cons.vars):
     """
     date = cons.convert_date(date) if date is not None else datetime.date.today()
     if date.strftime('%Y%m%d') not in calendar:
-        print('%s非交易日' %date.strftime('%Y%m%d'))
+        warnings.warn('%s非交易日' % date.strftime('%Y%m%d'))
         return None
     url = cons.DCE_RECIEPT_URL % (date.year, date.month - 1, date.day)
     data = pandas_readHtml_link(url, encoding='utf-8')[0]
@@ -49,7 +47,7 @@ def get_dce_reciept(date = None,vars=cons.vars):
         if type(x[0]) == type('a'):
             if x[0][-2:] == '小计':
                 var = x[0][:-2]
-                D = {'var':chinese_to_english(var),'reciept':int(x[3]),'date':date.strftime('%Y%m%d')}
+                D = {'var':chinese_to_english(var),'reciept':int(x[3]),'reciept_chg':int(x[4]),'date':date.strftime('%Y%m%d')}
                 records = records.append(pd.DataFrame(D,index=[0]))
     if len(records.index) != 0:
         records.index = records['var']
@@ -78,7 +76,7 @@ def get_shfe_reciept_1(date = None,vars = cons.vars):
     """
     date = cons.convert_date(date).strftime('%Y%m%d') if date is not None else datetime.date.today()
     if date not in calendar:
-        print('%s非交易日' %date)
+        warnings.warn('%s非交易日' % date.strftime('%Y%m%d'))
         return None
     if date == '20100126':
         shfe_20100126['date']=date
@@ -86,6 +84,9 @@ def get_shfe_reciept_1(date = None,vars = cons.vars):
     elif date == '20101029':
         shfe_20101029['date'] = date
         return shfe_20101029
+    elif date in ['20100416','20130821']:
+        print(u'20100416、20130821日期交易所数据丢失')
+        return None
     else:
         varList = ['天然橡胶', '沥青仓库', '沥青厂库', '热轧卷板', '燃料油', '白银', '线材', '螺纹钢', '铅', '铜', '铝', '锌', '黄金', '锡', '镍']
         url = cons.SHFE_RECIEPT_URL_1 % date
@@ -104,6 +105,7 @@ def get_shfe_reciept_1(date = None,vars = cons.vars):
             D={}
             D['var'] = chinese_to_english(dataCut[0].tolist()[0])
             D['reciept'] = int(dataCut[1].tolist()[-1])
+            D['reciept_chg'] = int(dataCut[2].tolist()[-1])
             D['date'] = date
             records = records.append(pd.DataFrame(D,index=[0]))
     if len(records.index) != 0:
@@ -133,7 +135,7 @@ def get_shfe_reciept_2(date = None,vars=None):
     """
     date = cons.convert_date(date).strftime('%Y%m%d') if date is not None else datetime.date.today()
     if date not in calendar:
-        print('%s非交易日' %date)
+        warnings.warn('%s非交易日' % date.strftime('%Y%m%d'))
         return None
     url = cons.SHFE_RECIEPT_URL_2 % date
     r = requests_link(url,encoding='utf-8')
@@ -148,7 +150,7 @@ def get_shfe_reciept_2(date = None,vars=None):
     records = pd.DataFrame()
     for var in set(data['VARNAME'].tolist()):
         dataCut = data[data['VARNAME'] == var]
-        D = {'var':chinese_to_english(re.sub("\W|[a-zA-Z]", "", var)),'reciept':int(dataCut['WRTWGHTS'].tolist()[-1]),'date':date}
+        D = {'var':chinese_to_english(re.sub("\W|[a-zA-Z]", "", var)),'reciept':int(dataCut['WRTWGHTS'].tolist()[-1]),'reciept_chg':int(dataCut['WRTCHANGE'].tolist()[-1]),'date':date}
         records = records.append(pd.DataFrame(D,index=[0]))
     if len(records.index) != 0:
         records.index = records['var']
@@ -175,8 +177,10 @@ def get_czce_reciept_1(date = None, vars=cons.vars):
     """
     date = cons.convert_date(date).strftime('%Y%m%d') if date is not None else datetime.date.today()
     if date not in calendar:
-        print('%s非交易日' %date)
+        warnings.warn('%s非交易日' % date.strftime('%Y%m%d'))
         return None
+    if date == '20090820':
+        return pd.DataFrame()
     url = cons.CZCE_RECIEPT_URL_1 % date
     r = requests_link(url,encoding='utf-8')
     r.encoding = 'utf-8'
@@ -197,9 +201,11 @@ def get_czce_reciept_1(date = None, vars=cons.vars):
             var = chinese_to_english(re.sub('[A-Z]+', '', dataCut[0].tolist()[0][3:]))
         if var == 'CF':
             reciept = dataCut[6].tolist()[-1]
+            reciept_chg = dataCut[7].tolist()[-1]
         else:
             reciept =dataCut[5].tolist()[-1]
-        D = {'var':var, 'reciept':int(reciept), 'date':date}
+            reciept_chg = dataCut[6].tolist()[-1]
+        D = {'var':var, 'reciept':int(reciept),'reciept_chg':int(reciept_chg), 'date':date}
         records = records.append(pd.DataFrame(D,index=[0]))
     if len(records.index) != 0:
         records.index = records['var']
@@ -225,7 +231,7 @@ def get_czce_reciept_2(date = None,vars = cons.vars):
     """
     date = cons.convert_date(date).strftime('%Y%m%d') if date is not None else datetime.date.today()
     if date not in calendar:
-        print('%s非交易日' %date)
+        warnings.warn('%s非交易日' % date.strftime('%Y%m%d'))
         return None
     url = cons.CZCE_RECIEPT_URL_2 % (date[:4], date)
     r = requests.get(url)
@@ -246,7 +252,8 @@ def get_czce_reciept_2(date = None,vars = cons.vars):
                 var = chinese_to_english(re.sub('[A-Z]+', '', string))
             dataCut.columns = dataCut.T[1].tolist()
             reciept = dataCut['仓单数量'].tolist()[-1]
-            D = {'var':var, 'reciept':int(reciept), 'date':date}
+            reciept_chg = dataCut['当日增减'].tolist()[-1]
+            D = {'var':var, 'reciept':int(reciept),'reciept_chg':int(reciept_chg), 'date':date}
             records = records.append(pd.DataFrame(D,index=[0]))
     if len(records.index) != 0:
         records.index = records['var']
@@ -273,7 +280,7 @@ def get_czce_reciept_3(date = None, vars = cons.vars):
 
     date = cons.convert_date(date).strftime('%Y%m%d') if date is not None else datetime.date.today()
     if date not in calendar:
-        print('%s非交易日' %date)
+        warnings.warn('%s非交易日' % date.strftime('%Y%m%d'))
         return None
     url = cons.CZCE_RECIEPT_URL_3 % (date[:4], date)
     r = requests_link(url,encoding='utf-8')
@@ -302,7 +309,8 @@ def get_czce_reciept_3(date = None, vars = cons.vars):
                 reciept = dataCut.loc[:, '仓单数量'].tolist()[-1]
             except:
                 reciept = dataCut.loc[:, '仓单数量(保税)'].tolist()[-1]
-            D = {'var': var, 'reciept': int(reciept), 'date': date}
+            reciept_chg = dataCut.loc[:, '当日增减'].tolist()[-1]
+            D = {'var': var, 'reciept': int(reciept),'reciept_chg':int(reciept_chg), 'date': date}
             records = records.append(pd.DataFrame(D, index=[0]))
     if len(records.index) != 0:
         records.index = records['var']
@@ -332,35 +340,47 @@ def get_reciept(start=None, end=None, vars=cons.vars):
     records=pd.DataFrame()
     while start <= end:
         if start.strftime('%Y%m%d') not in calendar:
-            print('%s非交易日' % start.strftime('%Y%m%d'))
+            warnings.warn('%s非交易日' % start.strftime('%Y%m%d'))
         else:
             print(start)
             for market,marketVars in cons.market_var.items():
 
                 if market == 'dce':
-                    f = get_dce_reciept
+                    if start >= datetime.date(2009,4,7):
+                        f = get_dce_reciept
+                    else:
+                        print(u'20090407起，dce每交易日更新仓单数据')
+                        f = None
                 elif market == 'shfe':
-                    if start <= datetime.date(2014,5,16):
+                    if start <= datetime.date(2014,5,16) and start >= datetime.date(2008,10,6):
                         f = get_shfe_reciept_1
-                    else:
+                    elif start > datetime.date(2014,5,16):
                         f = get_shfe_reciept_2
-                elif market == 'czce':
-                    if start <= datetime.date(2010,8,24):
-                        f = get_czce_reciept_1
-                    elif start <= datetime.date(2015,11,11):
-                        f = get_czce_reciept_2
                     else:
+                        f=None
+                        print(u'20081006起，shfe每交易日更新仓单数据')
+
+                elif market == 'czce':
+                    if start <= datetime.date(2010,8,24) and start >= datetime.date(2008,3,3):
+                        f = get_czce_reciept_1
+                    elif start <= datetime.date(2015,11,11) and start > datetime.date(2010,8,24):
+                        f = get_czce_reciept_2
+                    elif start > datetime.date(2015,11,11):
                         f = get_czce_reciept_3
+                    else:
+                        f=None
+                        print(u'20080303起，czce每交易日更新仓单数据')
 
 
                 get_vars = [var for var in vars if var in marketVars]
 
                 if market != 'cffex' and get_vars != []:
-                    records = records.append(f(start,get_vars))
+                    if f is not None:
+                        records = records.append(f(start,get_vars))
 
         start += datetime.timedelta(days=1)
     return records.reset_index(drop=True)
 
 if __name__ == '__main__':
-    d = get_czce_reciept_2(date='20141218')
+    d = get_reciept('20181128')
     print(d)
